@@ -399,7 +399,7 @@ def ensure_present(params, check_mode):
         if sg_cpc.uri != cpc.uri:
             raise ParameterError(
                 "Storage group {!r} is not associated with the specified "
-                "CPC %r, but with CPC %r.".
+                "CPC {!r}, but with CPC {!r}.".
                 format(storage_group_name, cpc.name, sg_cpc.name))
 
         try:
@@ -440,7 +440,7 @@ def ensure_present(params, check_mode):
             create_props, update_props = \
                 process_properties(cpc, storage_group, storage_volume, params)
             assert not create_props, \
-                "Unexpected create_props: %r" % create_props
+                "Unexpected create_props: {!r}".format(create_props)
             if update_props:
                 if not check_mode:
                     storage_volume.update_properties(update_props)
@@ -496,7 +496,7 @@ def ensure_absent(params, check_mode):
         if sg_cpc.uri != cpc.uri:
             raise ParameterError(
                 "Storage group {!r} is not associated with the specified "
-                "CPC %r, but with CPC %r.".
+                "CPC {!r}, but with CPC {!r}.".
                 format(storage_group_name, cpc.name, sg_cpc.name))
 
         try:
@@ -550,7 +550,7 @@ def facts(params, check_mode):
         if sg_cpc.uri != cpc.uri:
             raise ParameterError(
                 "Storage group {!r} is not associated with the specified "
-                "CPC %r, but with CPC %r.".
+                "CPC {!r}, but with CPC {!r}.".
                 format(storage_group_name, cpc.name, sg_cpc.name))
 
         try:
